# The GIMP -- an image manipulation program
# Copyright (C) 1995 Spencer Kimball and Peter Mattis
# -------------------------------------------------------------------------------------
# Translucent Text Effects python script (version 1.00) for GIMP 2.8
# Copyright (c) 2016 KGH
# -------------------------------------------------------------------------------------
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# -------------------------------------------------------------------------------------
# This file is a Python plug-in for GIMP v2.8. It has been tested on Windows 7 and 10.
# It can be executed by selecting the menu option: 'Filters/TextFX/Translucent Texts...'
# -------------------------------------------------------------------------------------
# Original aims
#
# Write a script to:
#Create a number of translucent text styles
# V. 1.00 Texts
# Simple Transparent Text
# Plastic Film Text
# Popped-out Text
# Frosted Text
# Shadow Text
# -------------------------------------------------------------------------------------

from gimpfu import *
import math

def proc_fx(img, layer, text_style, caption_text, font_name, font_size, font_colour, font_xpos, font_ypos, drop_shadowflag, clipshadow_flag, drop_x_offset, drop_y_offset, drop_blur, drop_colour, drop_opacity, bevel_size, bevel_depth):

        (mf_width, mf_height, mf_ascent, mf_descent) = pdb.gimp_text_get_extents_fontname(caption_text, font_size, PIXELS, font_name)
        image_width = float(img.width)
        image_height = float(img.height)
		
        
        # Some colors we may need
        black = gimpcolor.RGB(0, 0, 0)
        white = gimpcolor.RGB(255, 255, 255)
        darkgrey = gimpcolor.RGB(50, 50, 50)
        midgrey = gimpcolor.RGB(125, 125, 125)
        lightgrey = gimpcolor.RGB(170, 170, 170)
        #clipshadow_flag = True
	
	# Check for no caption or just spaces and (caption_text.isspace() == False)
   
        if ( (caption_text <> "") and (caption_text.isspace() == False) ):

        #REM: Select text style
            if (text_style == 0):
                proc_style_simple_transparent(img, caption_text, font_xpos, font_ypos, font_size, mf_height, font_name, mf_descent, image_height, image_width, drop_shadowflag, drop_x_offset, drop_y_offset, drop_blur, drop_colour, drop_opacity, bevel_size, bevel_depth, clipshadow_flag)
		
            if (text_style == 1):
                proc_style_plastic(img, layer, font_name, font_size, font_xpos, font_ypos, mf_height, mf_descent, caption_text, drop_shadowflag, drop_x_offset, drop_y_offset, drop_blur, drop_colour, drop_opacity, clipshadow_flag)

            if (text_style == 2):
                proc_frosted(img, caption_text, font_xpos, font_ypos, font_size, font_colour, mf_height, font_name, mf_descent, image_height, image_width, drop_shadowflag, drop_x_offset, drop_y_offset, drop_blur, drop_colour, drop_opacity, clipshadow_flag)

            if (text_style == 3):
                proc_popped(img, caption_text, font_xpos, font_ypos, font_size, mf_height, font_name, mf_descent, image_height, image_width, drop_shadowflag, drop_x_offset, drop_y_offset, drop_blur, drop_colour, drop_opacity, bevel_size, bevel_depth, clipshadow_flag)

            if (text_style == 4):
                proc_justshadow(img, caption_text, font_xpos, font_ypos, font_size, mf_height, font_name, mf_descent, image_height, image_width, drop_shadowflag, drop_x_offset, drop_y_offset, drop_blur, drop_colour, drop_opacity, bevel_size, bevel_depth, clipshadow_flag)
        else:

                
                old_handler = pdb.gimp_message_get_handler()		
                pdb.gimp_message_set_handler(MESSAGE_BOX)
                pdb.gimp_message("You have either entered no caption or just empty spaces.\nPlease enter a caption for this filter to use.")	
                pdb.gimp_message_set_handler(old_handler)
			
# -------------------------------------------------------------------------------------

def proc_justshadow(image, txt, f_xpos, f_ypos, f_size, f_height, f_name, f_descent, i_height, i_width, drop_flag, dr_x, dr_y, dr_blur, dr_colour, dr_opacity, bevel_size, bevel_depth, clipshadow_flag):
    
    #REM: Just shadow/clipped shadow text
	
    # Store the GIMP's settings so they can be restored
    gimp.context_push()

    # REM: Set start undo point
    pdb.gimp_image_undo_group_start(image)
    drop_flag = True
	
	# Some colors we may need
    black = gimpcolor.RGB(0, 0, 0)
    white = gimpcolor.RGB(255, 255, 255)
    darkgrey = gimpcolor.RGB(50, 50, 50)
    midgrey = gimpcolor.RGB(125, 125, 125)
    lightgrey = gimpcolor.RGB(170, 170, 170)

    # Rem: Get identifier of image layer for later use
    original_layer = pdb.gimp_image_get_active_layer(image)	

	# Rem: Create new text layer from UI settings
    text_layer = pdb.gimp_text_fontname(image, None, f_xpos, (i_height - (f_size + f_ypos) + f_descent), str(txt), -1, True, f_size, PIXELS, str( f_name ))
	
	# Rem: Set text colour to black
    pdb.gimp_text_layer_set_color(text_layer, black)
	
    pdb.gimp_item_set_name(text_layer, "Caption Layer")
    
    if (drop_flag == True):
        proc_dropshadow(image, text_layer, "Shadow Caption", "Text Channel", "Shadow Channel", dr_x, dr_y, str(txt), str( f_name ), f_size, dr_colour, dr_blur, dr_opacity, clipshadow_flag)

    pdb.gimp_image_remove_layer(image, text_layer)

    
    #Rem: Get the new identifier of the text layer
    text_layer2=pdb.gimp_image_get_active_layer(image)
	#REM Find position of text_layer relative to image
    (layer_x1, layer_y1) = pdb.gimp_drawable_offsets(text_layer2)
	#REM and offset text_layer to make it pop out
    pdb.gimp_layer_set_offsets(text_layer2, layer_x1 - dr_x, layer_y1 - dr_y)
	
    # REM: Set end undo point
    pdb.gimp_image_undo_group_end(image)
	    
    # Restore the GIMP's settings
    gimp.context_pop()

    # -------------------------------------------------------------------------------------
			
def proc_popped(image, txt, f_xpos, f_ypos, f_size, f_height, f_name, f_descent, i_height, i_width, drop_flag, dr_x, dr_y, dr_blur, dr_colour, dr_opacity, bevel_size, bevel_depth, clipshadow_flag):

    #Rem: Inspired by Pop out Text Tutorial from LittleWebHut

    # Store the GIMP's settings so they can be restored
    gimp.context_push()

    # REM: Set start undo point
    pdb.gimp_image_undo_group_start(image)
	
	# Some colors we may need
    black = gimpcolor.RGB(0, 0, 0)
    white = gimpcolor.RGB(255, 255, 255)
    darkgrey = gimpcolor.RGB(50, 50, 50)
    midgrey = gimpcolor.RGB(125, 125, 125)
    lightgrey = gimpcolor.RGB(170, 170, 170)

    # Rem: Get identifier of image layer for later use
    original_layer = pdb.gimp_image_get_active_layer(image)	

	# Rem: Create new text layer from UI settings
    text_layer = pdb.gimp_text_fontname(image, None, f_xpos, (i_height - (f_size + f_ypos) + f_descent), str(txt), -1, True, f_size, PIXELS, str( f_name ))
	
	# Rem: Set text colour to black
    pdb.gimp_text_layer_set_color(text_layer, black)
	
    pdb.gimp_item_set_name(text_layer, "Caption Layer")

    #REM Remove selection outline
    #pdb.gimp_selection_none(image)

    # Rem: set Text Layer as the active layer
    pdb.gimp_image_set_active_layer(image, text_layer)

    # Rem: Add alpha to selection
    pdb.gimp_image_select_item(image, 2, text_layer)	

	# Rem: Resize text layer (and rasterize) to image size
    temp = pdb.gimp_layer_resize_to_image_size(text_layer)
	
    #REM Copy from base image layer and paste into text outline
    temp = pdb.gimp_edit_copy(original_layer)
    pdb.gimp_image_set_active_layer(image, text_layer)
    temp = pdb.gimp_edit_paste(text_layer, True)
    pdb.gimp_floating_sel_anchor(temp)

    #REM Make Text Layer the active layer
    pdb.gimp_image_set_active_layer(image, text_layer)
 
	#Get pattern name for bevel filter even though we won't be using it
    pattern = pdb.gimp_patterns_get_list("")[1][0]
	
	#Settings for bevel filter
    pdb.python_layer_fx_bevel_emboss(image, text_layer, 2, bevel_depth, 0, bevel_size, 0, 120, 30, 0, white, SCREEN_MODE, 75, black, MULTIPLY_MODE, 75, 0, False, pattern, 1, 0, False, True)

    #Rem: Get the new identifier of the text layer
    text_layer=pdb.gimp_image_get_active_layer(image)

	#REM Find position of text_layer relative to image
    (layer_x1, layer_y1) = pdb.gimp_drawable_offsets(text_layer)
	#REM and offset text_layer to make it pop out
    pdb.gimp_layer_set_offsets(text_layer, layer_x1 - dr_x, layer_y1 - dr_y)

    #Rem: And merge with the shadow layer
    #pdb.gimp_image_merge_down(image, text_layer, CLIP_TO_BOTTOM_LAYER )

    if (drop_flag == True):
        proc_dropshadow(image, text_layer, "Shadow Caption", "Text Channel", "Shadow Channel", dr_x, dr_y, str(txt), str( f_name ), f_size, dr_colour, dr_blur, dr_opacity, clipshadow_flag)

    #Rem: Get the new identifier of the text layer
    text_layer=pdb.gimp_image_get_active_layer(image)
	
    #Rem: Rename the text layer
    pdb.gimp_item_set_name(text_layer, "Caption Layer")

    # REM: Set end undo point
    pdb.gimp_image_undo_group_end(image)
	    
    # Restore the GIMP's settings
    gimp.context_pop()
	
    # -------------------------------------------------------------------------------------
	
def proc_style_simple_transparent(image, txt, f_xpos, f_ypos, f_size, f_height, f_name, f_descent, i_height, i_width, drop_flag, dr_x, dr_y, dr_blur, dr_colour, dr_opacity, bevel_size, bevel_depth, clipshadow_flag):


    # Store the GIMP's settings so they can be restored
    gimp.context_push()

    # REM: Set start undo point
    pdb.gimp_image_undo_group_start(image)
	
 	
	# Define some colours we may need to use
    black = gimpcolor.RGB(0, 0, 0)
    white = gimpcolor.RGB(255, 255, 255)
    darkgrey = gimpcolor.RGB(50, 50, 50)
    midgrey = gimpcolor.RGB(125, 125, 125)
    lightgrey = gimpcolor.RGB(170, 170, 170)
	
    original_layer = pdb.gimp_image_get_active_layer(image)

	# Rem: Create new text layer
    #text_layer = pdb.gimp_text_fontname(image, None, 100, 800, "Doh", -1, True, 800, PIXELS, "Sans")
	# Rem: Create new text layer from UI settings
    text_layer = pdb.gimp_text_fontname(image, None, f_xpos, (i_height - (f_size + f_ypos) + f_descent), str(txt), -1, True, f_size, PIXELS, str( f_name ))
    pdb.gimp_item_set_name(text_layer, "Text Layer")
    
    # Rem: Set text colour to black
    pdb.gimp_text_layer_set_color(text_layer, midgrey)

    # Rem: set active layer
    pdb.gimp_image_set_active_layer(image, text_layer)

    # Rem: Resize text layer (and rasterize) to image size
    temp = pdb.gimp_layer_resize_to_image_size(text_layer)
	#### Crop text_layer to size of text layer
    pdb.plug_in_autocrop_layer(image, text_layer)

    #REM Remove selection outline
    pdb.gimp_selection_none(image)

	# Rem: set active layer
    pdb.gimp_image_set_active_layer(image, text_layer)	
	
	# Rem: temp blur text layer 
    #pdb.plug_in_gauss(image, text_layer, (5), (5), 1)	
	# Rem: blur text layer 
    blur_factor = 0.01	
    pdb.plug_in_gauss(image, text_layer, (f_size * blur_factor), (f_size * blur_factor), 1)
	
    #### Crop text_layer to size of text layer
    pdb.plug_in_autocrop_layer(image, text_layer)

	# REM Emboss
    #pdb.plug_in_emboss(image, text_layer, 46.0, 35.0, 5, 1)
    #pdb.plug_in_emboss(image, text_layer, 46.0, 35.0, 5, 1)

    #REM Select and add bevel
    pdb.gimp_image_select_item(image, 0, text_layer)
    #pdb.FU_selection_add_bevel(image, text_layer, 12, False, False, False)
    #pdb.script_fu_add_bevel(image, text_layer, 12, False, False)
	
    #Get pattern name for bevel filter even though we won't be using it
    pattern = pdb.gimp_patterns_get_list("")[1][0]
	#Settings for bevel filter
    #pdb.python_layer_fx_bevel_emboss(image, text_layer, 2, 13, 0, 40, 0, 120, 30, 0, white, SCREEN_MODE, 75, black, MULTIPLY_MODE, 75, 0, False, pattern, 1, 0, False, True)	
	#Settings for bevel filter
    pdb.python_layer_fx_bevel_emboss(image, text_layer, 2, bevel_depth, 0, bevel_size, 0, 120, 30, 0, white, SCREEN_MODE, 75, black, MULTIPLY_MODE, 75, 0, False, pattern, 1, 0, False, True)

    #Rem: Get the new identifier of the text layer
    text_layer=pdb.gimp_image_get_active_layer(image)
	
	# REM Set layer mode	
    pdb.gimp_layer_set_mode(text_layer, HARDLIGHT_MODE)

    if (drop_flag == True):
        proc_dropshadow(image, text_layer, "Shadow Caption", "Text Channel", "Shadow Channel", dr_x, dr_y, str(txt), str( f_name ), f_size, dr_colour, dr_blur, dr_opacity, clipshadow_flag)

    pdb.gimp_item_set_name(text_layer, "Caption Layer")
    #REM Remove selection outline
    pdb.gimp_selection_none(image)

    #pdb.gimp_image_set_active_layer(image, original_layer)

    # REM: Set end undo point
    pdb.gimp_image_undo_group_end(image)

    # Restore the GIMP's settings
    gimp.context_pop()
	
    # -------------------------------------------------------------------------------------

def proc_style_plastic(image, layer, f_name, f_size, f_xpos, f_ypos, f_height, f_descent, txt, drop_flag, dr_x, dr_y, dr_blur, dr_colour, dr_opacity, clipshadow_flag):
    #REM inspired by the Water text effect tutorial by Xgeous
    # Store the GIMP's settings so they can be restored
    gimp.context_push()

    # REM: Set start undo point
    pdb.gimp_image_undo_group_start(image)
	
    original_layer = pdb.gimp_image_get_active_layer(image)

    # REM: blur factor
    blur_factor = 0.03
	
    # Rem: Get image width and height
    image_width = float(image.width)
    image_height = float(image.height)
	
    # Rem: Ensure colours are black and white
    pdb.gimp_context_set_default_colors
    #### STEP 1
	# Rem: Create new text layer from UI settings
    text_layer = pdb.gimp_text_fontname(image, None, f_xpos, (image_height - (f_size + f_ypos) + f_descent), str(txt), -1, True, f_size, PIXELS, str( f_name ))

    #REM Rename the new text layer
    pdb.gimp_item_set_name(text_layer, "Caption Layer")

    # Rem: Set text colour to white
    pdb.gimp_text_layer_set_color(text_layer, (255, 255, 255))
	
    #### STEP 2
	# Rem: Resize text layer (and rasterize) to image size
    temp = pdb.gimp_layer_resize_to_image_size(text_layer)
	
    #### STEP 3
    # Rem: Add alpha to selection
    pdb.gimp_image_select_item(image, 2, text_layer)
    #temp = pdb.gimp_edit_copy(text_layer)
	
	
    #### STEP 3a Duplicate text_layer	
    text_layer2 = pdb.gimp_layer_copy(text_layer, False)
    pdb.gimp_image_insert_layer(image, text_layer2, None, 1)
    #REM Rename the new text layer
    pdb.gimp_item_set_name(text_layer, "Caption 2 Layer")
    pdb.gimp_item_set_linked(text_layer, True)
	
    #### STEP 4
    pdb.gimp_image_set_active_layer(image, text_layer)

    #### STEP 5	
	#Rem: Insert and display new channel 
    new_channel = pdb.gimp_selection_save(image)	
    pdb.gimp_image_set_active_channel(image, new_channel)
    pdb.gimp_channel_set_opacity(new_channel, 100)	
    pdb.gimp_item_set_visible(new_channel, True)	
    pdb.gimp_item_set_name(new_channel, "Channel")

    #### STEP 6
    pdb.plug_in_gauss(image, new_channel, (f_size * blur_factor), (f_size * blur_factor), 1)
		
    #### STEP 6a Duplicate channel
    new_channel2 = pdb.gimp_channel_copy(new_channel)
    pdb.gimp_image_insert_channel(image, new_channel2, None, 1)
	
    #### STEP 7
    pdb.gimp_item_set_visible(new_channel, False)	
    pdb.gimp_item_set_visible(new_channel2, False)
	
    #### STEP 8
    pdb.gimp_image_set_active_layer(image, text_layer)
	
    #### STEP 9a Set lighting effect to text_layer using new_channel for bump map
    pdb.plug_in_lighting(image, text_layer, new_channel, None, True, False, 0, 1, (255,255,255), -1.00, -1.00, 1.00, 1.87, -1.52, 1.00, 0.50, 0.25, 0.10, 0.005, 0.10, True, False, False)
	
	#### STEP 9b Set lighting effect to text_layer2 using new_channel2 for bump map
    pdb.gimp_image_set_active_layer(image, text_layer2)
    pdb.plug_in_lighting(image, text_layer2, new_channel2, None, True, False, 0, 1, (255,255,255), -1.00, -1.00, 1.00, -1.57, 1.51, 1.00, 0.50, 0.25, 0.10, 0.005, 0.10, True, False, False)

    #### STEP 10 Set layer mode for text_layer to grain-merge, opacity to 75% and merge down
    pdb.gimp_layer_set_mode(text_layer, GRAIN_MERGE_MODE)
    pdb.gimp_layer_set_opacity(text_layer, 75)


    pdb.gimp_layer_set_mode(text_layer2, GRAIN_MERGE_MODE)
    #text_layer = pdb.gimp_image_merge_down(image, text_layer, 0)
    pdb.gimp_item_set_linked(text_layer2, True)
	
    #### Step 11 set transparency colour
    pdb.plug_in_colortoalpha(image, text_layer,(222,222,222))
	
	#### Step 12 Final blur
    pdb.plug_in_blur(image, text_layer)

	#### Step 13 Crop text_layer
    pdb.plug_in_autocrop_layer(image, text_layer)
    #REM Rename the new text layer
    pdb.gimp_item_set_name(text_layer2, "Caption Layer")
	
	# Select text_layer
    pdb.gimp_image_set_active_layer(image, text_layer)
	
	# REM Drop Shadow
    if (drop_flag == True):
        #pdb.script_fu_drop_shadow(image, text_layer, dr_x, dr_y, dr_blur, dr_colour, dr_opacity, 1)
        proc_dropshadow(image, text_layer2, "Shadow Caption", "Text Channel", "Shadow Channel", dr_x, dr_y, str(txt), str( f_name ), f_size, dr_colour, dr_blur, dr_opacity, clipshadow_flag)
        temp = pdb.gimp_image_get_layer_by_name(image, "Shadow Caption")
        pdb.gimp_image_lower_item(image, temp)
        pdb.gimp_image_lower_item(image, temp)

	#### Step 14 Deselect 	
    pdb.gimp_selection_none(image)
	
    pdb.gimp_image_set_active_layer(image, original_layer)
	
    # REM: Set end undo point
    pdb.gimp_image_undo_group_end(image)
		
    # -------------------------------------------------------------------------------------

def proc_lighting(light_image, light_drawable, b_map, e_map, b_map_flag, e_map_flag, b_map_curve, l_source, l_colour, l_pos_x, l_pos_y, l_pos_z, l_dir_x, l_dir_y, l_dir_z, l_glow, l_bright, l_diffuse_reflec, l_shiny, l_polished, a_aliasing, n_image, mk_transp_bg):
    
    # REM: LIGHT 1 

    pdb.plug_in_lighting(light_image, light_drawable, b_map, e_map,  b_map_flag, e_map_flag, b_map_curve, l_source, l_colour, l_pos_x, l_pos_y, l_pos_z, l_dir_x, l_dir_y, l_dir_z, l_glow, l_bright, l_diffuse_reflec, l_shiny, l_polished, True, False, False)	

    # -------------------------------------------------------------------------------------	
 			
def proc_frosted(image, txt, f_xpos, f_ypos, f_size, f_colour, f_height, f_name, f_descent, i_height, i_width, drop_flag, dr_x, dr_y, dr_blur, dr_colour, dr_opacity, clipshadow_flag):

    # Rem: Inspired by the Ice Text Tutorial from LittleWebHut

    # Store the GIMP's settings so they can be restored
    gimp.context_push()

    # REM: Set start undo point
    pdb.gimp_image_undo_group_start(image)
	
 	
    original_layer = pdb.gimp_image_get_active_layer(image)

    # Rem: Create new text layer from UI settings
    text_layer = pdb.gimp_text_fontname(image, None, f_xpos, (i_height - (f_size + f_ypos) + f_descent), str(txt), -1, True, f_size, PIXELS, str(f_name))
	
    #Set feather amount to % of font size
    feather = int(f_size / 30)
	
	# Rem: Set text colour
    pdb.gimp_text_layer_set_color(text_layer, f_colour)
	
    pdb.gimp_item_set_name(text_layer, "Text_Layer")
	
    # Rem: Add alpha to selection Step 2
    pdb.gimp_image_select_item(image, CHANNEL_OP_REPLACE, text_layer)
	
    pdb.gimp_image_set_active_layer(image, text_layer)
	
    # Rem: Feather the selection Step 3	
    pdb.gimp_selection_feather(image, feather)
	
    pdb.gimp_image_set_active_layer(image, text_layer)
	
    # Rem: Step 3a	
    pdb.gimp_item_set_visible(text_layer, False)
	
    #REM: Step 4 Create new text layer - insert it - clear pixels to transparency
    text_layer2 = pdb.gimp_layer_new(image, i_width, i_height, RGBA_IMAGE, "Text_Layer2", 100, NORMAL_MODE)
    temp = pdb.gimp_image_insert_layer(image, text_layer2, None, -1)
    pdb.gimp_edit_clear(text_layer2)

    #REM: Step 5 Set FG fill and fill selection
    pdb.gimp_context_set_foreground(f_colour)
    pdb.gimp_edit_fill(text_layer2, FOREGROUND_FILL)

    #REM: Step 6 Deselect
    pdb.gimp_selection_none(image)

    # Step 7 Select by Colour
    pdb.gimp_image_select_color(image, CHANNEL_OP_REPLACE, text_layer2, f_colour)

    #Rem: Save large text outline selection for use later
    large_text_outline = pdb.gimp_selection_save(image)

    # Step 8 Was invert fuzzy select

    # Step 9 Create new text layer - insert it - clear pixels to transparency
    text_layer3 = pdb.gimp_layer_new(image, i_width, i_height, RGBA_IMAGE, "Text_Layer3", 100, NORMAL_MODE)
    temp = pdb.gimp_image_insert_layer(image, text_layer3, None, -1)
    pdb.gimp_edit_clear(text_layer3)

    # Step 10 Set FG fill and fill selection
    pdb.gimp_context_set_foreground(f_colour)
    pdb.gimp_edit_fill(text_layer3, FOREGROUND_FILL)

    # Step 11 Duplicate layer 
    text_layer4 = pdb.gimp_layer_copy(text_layer3, True)
    temp = pdb.gimp_image_insert_layer(image, text_layer4, None, -1)	
    pdb.gimp_item_set_name(text_layer4, "Text_Layer4")

	
    # Rem: Step 11a
    pdb.gimp_item_set_visible(text_layer2, False)
    pdb.gimp_item_set_visible(text_layer3, False)

    # Rem: Step 12 Set active layer
    pdb.gimp_image_set_active_layer(image, text_layer4)

    # Rem: Step 13 Select alpha layer from initial text layer
    pdb.gimp_image_select_item(image, CHANNEL_OP_REPLACE, text_layer)
	
    # Rem: Step 14 Feather the selection 	
    pdb.gimp_selection_feather(image, feather)

    # Rem: Step 15 Set active layer
    pdb.gimp_image_set_active_layer(image, text_layer4)

    # Rem: Step 16 Cut inside of the selection
    temp = pdb.gimp_edit_cut(text_layer4)

    pdb.gimp_selection_none(image)

    #pdb.gimp_image_merge_down(image, text_layer4, CLIP_TO_IMAGE)

    pdb.gimp_image_remove_layer(image, text_layer3)
    pdb.gimp_image_remove_layer(image, text_layer2)
    pdb.gimp_image_remove_layer(image, text_layer)

    pdb.gimp_layer_set_opacity(text_layer4, dr_opacity)

    #REM: Step 17 Create new layer for cracking with Flame plugin
    # REM: Sadly does not work in scripts
    #cracked_layer = pdb.gimp_layer_new(image, i_width, i_height, RGBA_IMAGE, "cracked_layer", 100, SCREEN_MODE)
    #pdb.gimp_image_insert_layer(image, cracked_layer, None, -1)
    #pdb.plug_in_flame(1, image, text_layer4)
	
    #REM: Step 17 Alternative plugins to hopefully add more realism
    pdb.plug_in_randomize_slur(image, text_layer4, 50, 3, TRUE, 20)
    pdb.plug_in_spread(image, text_layer4, 8, 8)
	#REM: Rotate layer to use Wind Plugin in the direction required, then rotate the layer back
    temp = pdb.gimp_item_transform_rotate (text_layer4, math.radians(270), False, 0, 0)
    pdb.plug_in_wind(image, text_layer4, (f_size / 10), 0, 10, 1, 1)
    temp = pdb.gimp_item_transform_rotate (text_layer4, math.radians(-270), False, 0, 0)

	#### Crop text_layer
    pdb.plug_in_autocrop_layer(image, text_layer4)

	
    #Rem: Drop Shadow if user requires - not a good look IMHO
    if (drop_flag == True):
        proc_dropshadow(image, text_layer4, "Shadow Caption", "Text Channel", "Shadow Channel", dr_x, dr_y, str(txt), str( f_name ), f_size, dr_colour, dr_blur, dr_opacity, clipshadow_flag)

     #REM: Rename layer
    pdb.gimp_item_set_name(text_layer4, "Caption Layer")

    #REM Remove selection outline
    pdb.gimp_selection_none(image)


    pdb.gimp_image_set_active_layer(image, original_layer)

    # REM: Set end undo point
    pdb.gimp_image_undo_group_end(image)
	    
    # Restore the GIMP's settings
    gimp.context_pop()
	
    # -------------------------------------------------------------------------------------

def proc_dropshadow(im, t_layer, t_layer2_name, chann_name, chann2_name, sh_xpos, sh_ypos, sh_text, sh_fname, sh_fsize, sh_colour, sh_blur, sh_op, clip_flag):
    #REM Create a text drop shadow effect - optionally clipped to the text layer for transparent type texts
	
    #REM: Select Text outline via alpha channel
    pdb.gimp_image_select_item(im, 2, t_layer)
	
    #REM: Create new text layer - insert it - clear pixels to transparency
    text_layer2 = pdb.gimp_layer_new(im, 10, 10, RGBA_IMAGE, "Text_Layer2", 100, NORMAL_MODE)
    temp = pdb.gimp_image_insert_layer(im, text_layer2, None, -1)

    #REM Resize new text layer
    temp = pdb.gimp_layer_resize_to_image_size(text_layer2)
    pdb.gimp_edit_clear(text_layer2)

	#REM Rename the new text layer
    pdb.gimp_item_set_name(text_layer2, str(t_layer2_name))
	
    #Rem: Fill selection with shadow colour
    pdb.gimp_context_set_foreground(sh_colour)
    pdb.gimp_edit_bucket_fill(text_layer2, FG_BUCKET_FILL, NORMAL_MODE, 100,0, FALSE, 0, 0)
	#REM Apply gaussian blur as per UI
    pdb.plug_in_gauss(im, text_layer2, sh_blur, sh_blur, 1)

    pdb.gimp_image_set_active_layer(im, text_layer2)
    temp = pdb.gimp_layer_translate(text_layer2, sh_xpos, sh_ypos)
	
    # REM create shadow layer channel and save it to a selection
    pdb.gimp_image_select_item(im, 0, text_layer2)
	
    #REM get bounds of this selection before altering 
    #(shadow, shad_x1, shad_y1, shad_x2, shad_y2) = pdb.gimp_selection_bounds(im)
    shadow_channel = pdb.gimp_selection_save(im)
    pdb.gimp_item_set_name(shadow_channel, str(chann2_name))

    # REM: Check to see what type of shadow is required
    if (clip_flag == True):
        # Rem: Add alpha to selection and subtract it from shadow layer
        pdb.gimp_image_select_item(im, 1, t_layer)
    else:
        # Rem: Add alpha to selection and add it to the shadow layer
        pdb.gimp_image_select_item(im, 0, t_layer)

    #REM: Save the selection and give it a name
    t_channel = pdb.gimp_selection_save(im)
    pdb.gimp_item_set_name(t_channel, str(chann_name))
	
    #REM Invert selection
    pdb.gimp_selection_invert(im)
	
    # Rem: set active layer
    pdb.gimp_image_set_active_layer(im, text_layer2)
	#REM Clear the inverted selection
    pdb.gimp_edit_clear(text_layer2)
	
    #pdb.gimp_layer_resize(text_layer2, shad_x2 - shad_x1, shad_y2 - shad_y1, 0 - shad_x1, 0 - shad_y1)
	
    # REM Layer opacity	
    pdb.gimp_layer_set_opacity(text_layer2, sh_op)
	
	#REM Clear Selection
    pdb.gimp_selection_none(im)
	
	#REM Crop layers
    pdb.gimp_image_set_active_layer(im, text_layer2)
    pdb.plug_in_autocrop_layer(im, text_layer2)
    pdb.gimp_image_set_active_layer(im, t_layer)
    pdb.plug_in_autocrop_layer(im, t_layer)

    temp = pdb.gimp_image_raise_item (im, t_layer)

    #REM link layers together
    pdb.gimp_item_set_linked(t_layer, True)
    pdb.gimp_item_set_linked(text_layer2, True)
	
   



register(
    "python_fu_translucent_text",
    "Adds a number of translucent text effects",
    "Adds a number of translucent text effects as plug-ins",
    "KGH",
    "Skinnyhouse Enterprises",
    "2016",
    "<Image>/Filters/Text FX/Translucent Texts...",
    "*",
    [
    (PF_OPTION, "pf_option", "Text effect", 0, ("Simple Transparent", "Plastic Film", "Frosted", "Popped-out", "Plain Shadow")),
    (PF_STRING, "pf_string", "Caption text:", str( unichr(169))),
    (PF_FONT, "pf_fontnamedialog", "Font:", "Sans"),
    (PF_SPINNER, "pf_fontsize", "Font size:", 800, (1, 3000, 1)),
    (PF_COLOR, "pf_color", "Text colour:", (0.0, 0.0, 0.0)),
    (PF_INT, "pf_fontxpos", "Text position (from left):", 80),
    (PF_INT, "pf_fontypos", "Text position (from bottom):", 40),
    (PF_TOGGLE, "pf_dropshadowonoff",   "Add a drop shadow", 0),
    (PF_TOGGLE, "pf_clipflagonoff",   "Clip the shadow", 0),
    (PF_SPINNER, "pf_xoffset", "Shadow x-offset:", 8, (1, 3000, 1)),
    (PF_SPINNER, "pf_yoffset", "Shadow y-offset:", 8, (1, 3000, 1)),
    (PF_SPINNER, "pf_bluroffset", "Shadow blur radius:", 15, (1, 100, 1)),
    (PF_COLOR, "pf_shadowcolour", "Shadow Colour:", (0.0, 0.0, 0.0)),
    (PF_SLIDER, "pf_shadowopacity", "Shadow Opacity:", 75, (0, 100, 1)),
    (PF_SLIDER, "pf_bevelsize", "Bevel Size:", 10, (0, 250, 1)),
    (PF_SLIDER, "pf_beveldepth", "Bevel Depth:", 50, (0, 65, 1)),
    ],
    [],
    proc_fx)



main()